int Alea (short min, short max)
{
	return (unsigned short)rand() * (max - min) / 65536 + min;
}


// ****************************************************************************
// ****************************** PrintGrille *********************************
// ****************************************************************************
void PrintGrille (GRILLE grille)
{
	int x, y;
	
	for (y = 0; y < grille.xc; y ++)
    {
		for (x = 0; x < grille.yc; x ++)
			printf (" %d", grille.c[x][y]);
		printf ("\n");
    }
	printf ("\n");
}


// ****************************************************************************
// ****************************** ImageErreur *********************************
// ****************************************************************************
void ImageErreur (char *image)
{
	// Affiche un message d'erreur et quitte le programme
	fprintf (stderr, "Impossible de charger l'image %s (%s)\n", image, SDL_GetError ());
	exit (1);
}


// ****************************************************************************
// ****************************** Case2Coord **********************************
// ****************************************************************************
int Case2CoordX (int x, GRILLE grille)
{
	// Renvoit l'abscisse cran du coin haut gauche de la case
	return grille.x1 + x * grille.tc;
}

int Case2CoordY (int y, GRILLE grille)
{
	// Renvoit l'abscisse cran du coin haut gauche de la case
	return grille.y1 + y * grille.tc;
}


// ****************************************************************************
// ****************************** Clic2Case ***********************************
// ****************************************************************************
int Clic2CaseX (int x, GRILLE grille)
{
	// Renvoit l'abscisse de la case clique ou -1 si en dehors de la grille
	if (x >= grille.x1 && x < grille.x2)
		return (x - grille.x1 - 1) / grille.tc;
	else
		return -1;
}

int Clic2CaseY (int y, GRILLE grille)
{
	// Renvoit l'ordonne de la case clique ou -1 si en dehors de la grille
	if (y >= grille.y1 && y <= grille.y2)
		return (y - grille.y1 - 1) / grille.tc;
	else
		return -1;
}


// ****************************************************************************
// ******************************* Attendre_FPS *******************************
// ****************************************************************************
void Attendre_FPS (void)
{
	static int temps = 0;				// Moment du dernier appel de cette fonction
	
	// Marque une pause pour synchroniser l'affichage sur la vitesse choisie pour que
	// le temps entre 2 appels de cette fonction soit de: 1000 ms / Frames Par Seconde
	while (SDL_GetTicks() < temps + 1000 / FPS);
	
	temps = SDL_GetTicks ();			// Moment o cette fonction se termine
}


// ****************************************************************************
// ****************************** Secouer_Ecran *******************************
// ****************************************************************************
void Secouer_Ecran (JOUEUR j[], GRILLE grille, SDL_Surface *ecran)
{
	SDL_Rect fond, rect, src;
	int xDecal, yDecal;
	int i;
  
	// Rectangle de copie du fond d'cran
	fond.x = grille.x1 - (j[0].imgCours->w / j[0].nCours - grille.imgCase->w) / 2 - 16;
	fond.y = grille.y1 - (j[0].imgCours->h / 4           - grille.imgCase->h) / 2 - 16;
	fond.w = grille.x2 + (grille.x1 - fond.x) * 2 + 32;
	fond.h = grille.y2 + (grille.y1 - fond.y) * 2 + 32;
	// Rectangle de copie des cases
	rect.w = rect.h = grille.tc;
	// Rectangle de copie des images des joueurs
	src.x = src.y = 0;
	src.w = j[0].rect.w;
	src.h = j[0].rect.h;
	
	// Dplace alatoirement la grille
	for (i = 0; i <= TREMBLE_NB / TREMBLE_VITESSE; i ++)
    {
		// Dcallage alatoire
		xDecal = Alea (-TREMBLE_DECAL, TREMBLE_DECAL);
		yDecal = Alea (-TREMBLE_DECAL, TREMBLE_DECAL);
		// Replace la grille  la position de dpart
		if (i == TREMBLE_NB / TREMBLE_VITESSE) xDecal = yDecal = 0;
		
		// Affiche le fond
		SDL_BlitSurface (grille.imgFond, &fond, ecran, &fond);
		
		// Affiche les cases l o il y en a, avec le dcallage du tremblement
		for (rect.x = grille.x1 + xDecal; rect.x < grille.x2 + xDecal; rect.x += rect.w)
			for (rect.y = grille.y1 + yDecal; rect.y < grille.y2 + yDecal; rect.y += rect.h)
				if (grille.c[(rect.x-xDecal-grille.x1)/rect.w][(rect.y-yDecal-grille.y1)/rect.h] != TROU)
					SDL_BlitSurface (grille.imgCase, NULL, ecran, &rect);
		
		// Affiche les joueurs avec le dcallage du tremblement
		j[0].rect.x += xDecal;
		j[0].rect.y += yDecal;
		j[1].rect.x += xDecal;
		j[1].rect.y += yDecal;
		
		SDL_BlitSurface (j[0].imgCours, &src, ecran, &j[0].rect);
		SDL_BlitSurface (j[1].imgCours, &src, ecran, &j[1].rect);
		
		// Rtablit les positions des joueurs sans le tremblement
		j[0].rect.x -= xDecal;
		j[0].rect.y -= yDecal;
		j[1].rect.x -= xDecal;
		j[1].rect.y -= yDecal;
		
		Attendre_FPS ();
		SDL_UpdateRects (ecran, 1, &fond);
	}
}


// ****************************************************************************
// ********************************* Dans_Rect ********************************
// ****************************************************************************
BOOL Dans_Rect (int x, int y, SDL_Rect rect)
{
	if (x >= rect.x && x <= (rect.x + rect.w) && y >= rect.y && y <= (rect.y + rect.h))
		return OUI;
	else
		return NON;
}


// ****************************************************************************
// ************************** Afficher_Caractere ******************************
// ****************************************************************************
void Afficher_Caractere (int x, int y, char c, POLICE police, SDL_Surface *ecran)
{
	SDL_Rect dest;

	// Place le rectangle sur la bonne image du caractre  afficher
	if (c >= 'A' && c <= 'Z') police.rect.x = (c - 'A') * police.rect.w;
	else if (c >= 'a' && c <= 'z') police.rect.x = (c - 'a') * police.rect.w;
	else if (c >= '0' && c <= '9') police.rect.x = (c - '0' + 26) * police.rect.w;
	else if (c == ':') police.rect.x = 36 * police.rect.w;
	else return;

	// Position sur l'cran
	dest.x = x;
	dest.y = y;
	dest.w = police.rect.w;
	dest.h = police.rect.h;

	// Copie le caractre et met  jour
	SDL_BlitSurface (police.img, &police.rect, ecran, &dest);
	SDL_UpdateRects (ecran, 1, &dest);
}


// ****************************************************************************
// **************************** Afficher_Chaine *******************************
// ****************************************************************************
void Afficher_Chaine (int x, int y, int vitesse, char chaine[], POLICE police, SDL_Surface *ecran, SDL_Surface *back)
{
	SDL_Rect dest = {0, 0, police.rect.w, police.rect.h};
	int xDep, yDep, xDest, yDest;
	int c;
	int frame;

	// Parcours la chaine de caractre et affiche les caractres un par un en
	// les faisant glisser de l'un des bord de l'cran
	for (c = 0; chaine[c] != '\0'; c ++)
	{
		// Choisit alatoirement le bord de l'cran et sa position dessus
		switch (Alea (0, 2))
		{
			case 0:	xDep = Alea (0, ecran->w - police.rect.w);
					yDep = Alea (0, 2) ? 0 : ecran->h - police.rect.h;
					break;
			case 1:	xDep = Alea (0, 2) ? 0 : ecran->w - police.rect.w;
					yDep = Alea (0, ecran->h - police.rect.h);
					break;
			default:	xDep = yDep = 0;
		}
		
		xDest = x + c * police.rect.w;
		yDest = y;
		
		// Fait glisser le caractre du bord de l'cran vers sa position finale
		if (vitesse > 0)
		for (frame = 0; frame <= vitesse; frame ++)
		{
			// Efface l'ancienne position
			SDL_BlitSurface (back, &dest, ecran, &dest);
			
			// Affiche le caractre au bon endroit
			Afficher_Caractere (xDep + ((xDest - xDep) * frame) / vitesse, yDep + ((yDest - yDep) * frame) / vitesse, chaine[c], police, ecran);
			SDL_UpdateRects (ecran, 1, &dest);
			
			// Fait la moyenne pour savoir o doit tre affich le caractre
			dest.x = xDep + ((xDest - xDep) * frame) / vitesse;
			dest.y = yDep + ((yDest - yDep) * frame) / vitesse;
			Attendre_FPS ();
		}

		// Affiche le caractre  sa position finale
		Afficher_Caractere (xDest, yDest, chaine[c], police, ecran);
		Afficher_Caractere (xDest, yDest, chaine[c], police, back);
	}
}
