// Tailles de la grille
#define XCASEMIN 4
#define YCASEMIN 4
#define XCASEMAX 8
#define YCASEMAX 8
#define MAXCASES 10

// Vitesse des animation
#define TREMBLE_NB 10			// Nombre d'itrations du tremblement de la grille
#define TREMBLE_DECAL 6			// Dcallage maximum du tremblement de la grille
#define TREMBLE_VITESSE 2		// Vitesse du tremblement
#define BOULE_VITESSE 2			// Vitesse de l'animation de la boule
#define CARACT_VITESSE 10		// Vitesse d'apparition des caractre

// Valeurs de jeu par dfaut
#define DFT_J1ROUGE NON
#define DFT_J1TYPE HUMAIN
#define DFT_J2TYPE HUMAIN
#define DFT_J1NIVO 2
#define DFT_J2NIVO 2
#define DFT_INTEMP NON
#define DFT_DOUBLE_DEPLACE NON
#define DFT_DOUBLE_BOULEFEU NON
#define DFT_XGRILLE 6
#define DFT_YGRILLE 6
#define DFT_FPS 96

// Textes
#define TXT_OUI			"YES"
#define TXT_NON			"NO"
#define TXT_SIMPLE		"SIMPLE"
#define TXT_DOUBLE		"DOUBLE"
#define TXT_J1NOM		"PLAYER 1"
#define TXT_J2NOM		"PLAYER 2"
#define TXT_HUMAIN		"HUMAN"
#define TXT_ORDI		"COMPUTER"
#define TXT_JOUER		"START"
#define TXT_INTEMP		"        MISC:"
#define TXT_DEPLACE		"        MOVE:"
#define TXT_BOULEFEU	"    FIREBALL:"
#define TXT_VITESSEJEU	"  GAME SPEED:"
#define TXT_TAILLEGRILLE "   GRID SIZE:   X"
#define TXT_J1GAGNE		"THE PLAYER 1 WIN"
#define TXT_J2GAGNE		"THE PLAYER 2 WIN"

// Fichiers des images
#define IMG_MENU		"menu.bmp"
#define IMG_FOND1		"fond1.bmp"
#define IMG_FOND2		"fond2.bmp"
#define IMG_FOND3		"fond3.bmp"
#define IMG_FOND4		"fond4.bmp"
#define IMG_FOND5		"fond5.bmp"
#define IMG_NIVO		"dossi-nivo.bmp"
#define IMG_POLICE		"police.bmp"
#define IMG_CASE		"case.bmp"
#define IMG_FEU			"feu.bmp"
#define IMG_J1COURS		"dossi-cours.bmp"
#define IMG_J2COURS		"dossa-cours.bmp"
#define IMG_J1LANCE		"dossi-lance.bmp"
#define IMG_J2LANCE		"dossa-lance.bmp"
#define IMG_J1GAGNE		"dossi-victoire.bmp"
#define IMG_J2GAGNE		"dossa-victoire.bmp"
#define IMG_J1PERDU		"dossi-ange.bmp"
#define IMG_J2PERDU		"dossa-ange.bmp"
#define IMG_J1ATTEND	"dossi-attend.bmp"
#define IMG_J2ATTEND	"dossa-attend.bmp"
#define IMG_J1ATTENDFEU	"dossi-attend-feu.bmp"
#define IMG_J2ATTENDFEU	"dossa-attend-feu.bmp"

// Vitesse d'affichage des animations en frames par seconde
int FPS;

// Types pour dsigner le contenu des cases, l'action en cours, ...
typedef enum {NON, OUI} BOOL;
typedef enum {VIDE, J1, J2, TROU} CASE;
typedef enum {J1_DEPLACE, J1_TROU, J2_DEPLACE, J2_TROU} ACTION;
typedef enum {HUMAIN, ORDI} TYPEJOUEUR;
typedef enum {COURSB, COURSH, COURSG, COURSD, LANCE, GAGNE, PERDU, ATTEND, ATTENDFEU} ANIM;

// Structure contenant une police de caractere
typedef struct {
	SDL_Rect rect;				// Rectangle de la taille d'un caractre
	SDL_Surface *img;			// Image contenant les caractres
} POLICE;

// Structure permettant de mmoriser les cases offrant le plus de dplacements
typedef struct {
  int x, y;						// Coordonnes de la case
  int nb;						// Valeur de la case
} MAX;

// Structure contenant les infos d'un joueur
typedef struct {
	CASE n;						// Numro du joueur
	TYPEJOUEUR type;			// Humain ou Ordi
	int nivo;					// Niveau du joueur
	int x, y;					// Position sur la grille
	SDL_Rect rect, src;			// Rectangle de position sur l'cran et dans l'image d'animation
	SDL_Surface *imgCours;		// Images des dplacements
	SDL_Surface *imgLance;		// Images du lancement de la boule de feu
	SDL_Surface *imgGagne;		// Images de la victoire
	SDL_Surface *imgPerdu;		// Images de la dfaite
	SDL_Surface *imgAttend;		// Images de l'attente du dplacement
	SDL_Surface *imgAttendFeu;	// Images de l'attente du lancement de la boule de feu
	SDL_Surface *imgFeu;		// Images de la boule de feu
	// Nombre d'images pour les animations
	int nCours, nLance, nGagne, nPerdu, nAttend, nAttendFeu, nFeu;
	ANIM anim;					// Animation en cours
	int frame;					// Image en cours dans l'animation
} JOUEUR;

// Structure contenant les infos sur la grille du jeu
typedef struct {
	int xc, yc;					// Nombre de cases en longueur et en hauteur
	int x1, y1;					// Position du coin haut gauche de la grille
	int x2, y2;					// Position du coin bas  droite de la grille
	int tc;						// Taille des cases
	int c[XCASEMAX][YCASEMAX];	// Grille indiquant le contenu des cases
	SDL_Surface *imgCase;		// Image d'une case
	SDL_Surface *imgFond;		// Image de l'arrire plan
} GRILLE;

// Structure contenant les infos gnrales sur le jeu
typedef struct {
	SDL_Surface *ecran;			// Surface graphique principale (fenetre du prog)
	SDL_Surface *back;			// Surface graphique tampon (back buffer)
	POLICE police;				// Police pour l'affichage de caractres
	BOOL J1ROUGE;				// Le joueur 1 est vert ou rouge ?
	BOOL INTEMP;				// Pluie de boule de feu ?
	BOOL DOUBLE_DEPLACE;		// Deplacement simple ou double ?
	BOOL DOUBLE_BOULEFEU;		// Place un ou deux trous ?
	TYPEJOUEUR J1TYPE;			// Humain ou Ordi ?
	TYPEJOUEUR J2TYPE;			// Humain ou Ordi ?
	int J1NIVO;					// Niveau de l'IA
	int J2NIVO;					// Niveau de l'IA
	int XGRILLE, YGRILLE;		// Taille de la grille
} JEU;

// Renvoie au hasard un nombre compris entre les valeurs passes en arguments (la 2e non incluse)
int Alea (short, short);

// Affiche dans la console le contenu de la grille
void PrintGrille (GRILLE);

// Affiche l'erreur obtenue pendant le chargement d'une image
void ImageErreur (char *);

// Convertit les coordonnes du clic de souris en position dans la grille
int Case2CoordX (int, GRILLE);
int Case2CoordY (int, GRILLE);

// Convertit la position dans la grille en coordonnes sur l'cran
int Clic2CaseX (int, GRILLE);
int Clic2CaseY (int, GRILLE);

// Synchronise le temps sur la vitesse d'affichage (FPS)
void Attend_FPS (void);

// Remplit l'arrire plan avec l'image passe en argument
void Creer_Fond (GRILLE *, SDL_Surface *);

// Fait trembler l'cran
void Secouer_Ecran (JOUEUR[], GRILLE, SDL_Surface *);

// Affiche le joueur avec l'image de l'animation en cours
void Afficher_Joueur (JOUEUR *, SDL_Surface *);

// Renvoie OUI si le trou a put tre plac (et place le trou)
BOOL Placer_Trou (int , int , JOUEUR[], int, GRILLE *, SDL_Surface *, SDL_Surface *);

// Renvoie OUI si le joueur a put tre dplac (et dplace le joueur)
BOOL Deplacer_Joueur (int, int, JOUEUR[], int, GRILLE *, SDL_Surface *, SDL_Surface *);

// Place le joueur  la position choisie et renvoie OUI si la position est disponible
BOOL Placer_Joueur (int, int, JOUEUR[], int, GRILLE *, SDL_Surface *, SDL_Surface *);

// Permet aux joueurs de choisir leur position de dpart
void Placer_Joueurs (JOUEUR[], GRILLE *, SDL_Surface *, SDL_Surface *);

// Cre la grille, met les images en mmoire, ...
void Constructeur (JOUEUR[], GRILLE *, BOOL, SDL_Surface *, SDL_Surface *);

// Libre la mmoire occupe par les images
void Destructeur (JOUEUR[], GRILLE *);

// Prvois les possibilits de dplacements pour chaques cases
void Calcul_Choix (GRILLE , int[][], int , int *);

// Cre une grille  partir du jeu pour calculer les dplacements possibles
void Meilleur_Choix (GRILLE , int[][], int, JOUEUR[], int, MAX[], MAX []);

// Place sur l'une des meilleurs cases
void Ordi_Place (int *, int *, JOUEUR, GRILLE);

// Dplace sur l'une des meilleures cases
void Ordi_Deplace (int *, int *, JOUEUR [], int, GRILLE);

// Dtruit l'une des meilleures cases de l'adversaire
void Ordi_Trou (int *, int *, JOUEUR [], int, GRILLE);

// Boucle de jeu principale qui gre l'affichage et les vnements (souris, ...)
void Jeu (JEU *);

// Affiche un caractre  l'cran
void Afficher_Caractere (int, int, char, POLICE, SDL_Surface *);

// Affiche une chaine de caractre en faisant apparaitre les caractres en
// glissan de l'un des bord de l'cran
void Afficher_Chaine (int, int, int, char[], POLICE, SDL_Surface *, SDL_Surface *);

// Affiche les images des personnages reprsentant les niveaux d'intelligence
void Afficher_IA (int, SDL_Rect[], SDL_Rect[], SDL_Surface *, SDL_Surface *, SDL_Surface *, SDL_Surface *);

// Menu de choix des options
void Menu (JEU *jeu);

// Fonction principale qui vrifie les paramtres passs  l'appel du programme,
// initialise la bibliothque graphique SDL, cre l'affichage, ...
int main (int, char *[]);
